package question;

use POSIX;

sub list {
	my $qid;
	my $file;
	my @list;

	@list = ();

	open(QUESTIONLIST, "./question/list.txt");
	while (<QUESTIONLIST>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($qid, $file) = split('\s+');
		push(@list, $qid);
	}
	close(QUESTIONLIST);

	return @list;
}

sub is_valid {
	my $search_qid;
	my $qid;
	my $file;

	($search_qid) = @_;
	$file = get_file($search_qid);
	if ($file eq "") {
		return 0;
	}
	return 1;
}

sub get_file {
	my $search_qid;
	my $qid;
	my $file;

	($search_qid) = @_;

	open(QUESTIONLIST, "./question/list.txt");
	while (<QUESTIONLIST>) {
		s/^\s+//;
		s/\s+$//;
		if (/^#/ || ($_ eq "")) {
			next;
		}
		($qid, $file) = split('\s+');
		if ($search_qid ne $qid) {
			next;
		}
		close(QUESTIONLIST);
		return $file;
	}
	close(QUESTIONLIST);

	return "";
}

sub get_genre {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = readfile($qid);
	return $genre;
}

sub get_score {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = readfile($qid);
	return $score;
}

sub get_title {
	my $qid;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;
	($genre, $score, $title, $question, @answers) = readfile($qid);
	return $title;
}

sub readfile {
	my $qid;
	my $file;
	my $key;
	my $value;
	my $in_question;
	my $in_comment;
	my $genre;
	my $score;
	my $title;
	my $question;
	my @answers;

	($qid) = @_;

	$file = get_file($qid);
	if ($file eq "") {
		return ();
	}

	$in_question = 0;
	$in_comment = 0;

	open(FILE, "./question/$file");
	while (<FILE>) {
		if ($in_question) {
			if (/^\s*QUESTION_END/) {
				$in_question = 0;
			} else {
				$question .= $_;
			}
			next;
		}
		if ($in_comment) {
			if (/^\s*COMMENT_END/) {
				$in_comment = 0;
			}
			next;
		}

		s/^\s+//;
		s/\s+$//;

		if (/^#/ || ($_ eq "")) {
			next;
		}

		($key, $value) = /(\w+):\s+(.+)/;

		if (/^GENRE/) {
			$genre = $value;
		}
		if (/^SCORE/) {
			$score = $value;
		}
		if (/^TITLE/) {
			$title = $value;
		}
		if (/^ANSWER/) {
			push(@answers, $value);
		}
		if (/^QUESTION/) {
			$in_question = 1;
		}
		if (/^COMMENT/) {
			$in_comment = 1;
		}
	}
	close(FILE);

	return ($genre, $score, $title, $question, @answers);
}

sub reply {
	my $uid;
	my $qid;
	my $reply;
	my $t;
	($uid, $qid, $reply) = @_;

	if (!-d "./result/$uid") {
		mkdir("./result/$uid");
		chmod(0777, "./result/$uid");
	}

	$t = time();
	open(RESULTFILE, ">> ./result/$uid/${qid}-${t}.txt");
	print RESULTFILE "$reply\n";
	print RESULTFILE "$t\n";
	$t = localtime($t);
	print RESULTFILE "$t\n";
	close(RESULTFILE);
}

sub clear {
	my $uid;
	my $qid;
	my $t;
	($uid, $qid) = @_;

	if (!-d "./result/$uid") {
		mkdir("./result/$uid");
		chmod(0777, "./result/$uid");
	}

	open(RESULTFILE, "> ./result/$uid/${qid}.txt");
	$t = time();
	print RESULTFILE "$t\n";
	$t = localtime($t);
	print RESULTFILE "$t\n";
	close(RESULTFILE);
}

sub is_clear {
	my $uid;
	my $qid;
	($uid, $qid) = @_;

	if (-f "./result/$uid/${qid}.txt") {
		return 1;
	}
	return 0;
}

sub get_time {
	my $uid;
	my $qid;
	($uid, $qid) = @_;

	if (!is_clear($uid, $qid)) {
		return 0;
	}

	open(RESULTFILE, "./result/$uid/${qid}.txt");
	$t = <RESULTFILE>;
	close(RESULTFILE);

	return $t + 0;
}

sub clear_list {
	my @qid_list;
	my $uid;
	my $qid;
	my @clear_list;

	($uid) = @_;

	@qid_list = list();
	@clear_list = ();

	foreach $qid (@qid_list) {
		if (is_clear($uid, $qid)) {
			push(@clear_list, $qid);
		}
	}

	return @clear_list;
}

sub total_score {
	my @qid_list;
	my $uid;
	my $qid;
	my $t;
	my $total_score = 0;

	($uid, $t) = @_;

	@qid_list = clear_list($uid);

	foreach $qid (@qid_list) {
		if (defined($t) && ($t > 0)) {
			if (get_time($uid, $qid) > $t) {
				next;
			}
		}
		$total_score += get_score($qid);
	}

	return $total_score;
}

1;
