#!/usr/bin/perl

use strict;
use CGI;
use CGI::Session;

require 'lib/cgilib.pl';
require 'lib/session.pl';
require 'lib/user.pl';
require 'lib/question.pl';

my $cgi = CGI->new;
my $session_id;
my $session;

($session_id, $session) = session::check($cgi);

my $uid = $session->param('uid');
if (!user::is_valid($uid)) {
	cgilib::header($cgi, "Error");
	print <<"EOD";
<h1>Invalid User!</h1>
<p><a href="login.cgi">Login</a></p>
EOD
	cgilib::footer($cgi);
	exit;
}

my $name = user::get_name($uid);

cgilib::header($cgi, "Progress");
cgilib::menu($session_id, $name);

print <<"EOD";
<canvas id="chart"></canvas>
<script src="moment.min.js"></script>
<script src="Chart.min.js"></script>
<script>
	var ctx = document.getElementById("chart").getContext('2d');
	var myChart = new Chart(ctx, {
		type: 'line',
		data: {
			datasets: [
EOD

	my @userdata;
	# gbv20[U[̂
	my %score;
	for my $uid (user::list()) {
		$score{$uid} = question::total_score($uid, time);
	}
	my @uid_list = sort { $score{$b} <=> $score{$a} } keys %score;
	splice @uid_list, 20;

	# gnuplot ɏzF
	my @color_list = qw(
		ff0000 00c000 0080ff c000ff 00eeee c04000 c8c800 4169e1 ffc020 008040
		c080ff 306080 8b0000 408000 ff80ff 7fffd4 a52a2a ffff00 40e0d0 000000
		1a1a1a 333333 4d4d4d 666666 7f7f7f 999999 b3b3b3 c0c0c0 cccccc e5e5e5
		f03232 90ee90 add8e6 f055f0 e0ffff eedd82 ffb6c1 afeeee ffd700 00ff00
		006400 00ff7f 228b22 2e8b57 0000ff 00008b 191970 000080 0000cd 87ceeb
		00ffff ff00ff 00ced1 ff1493 ff7f50 f08080 ff4500 fa8072 e9967a f0e68c
		bdb76b b8860b f5f5dc a08020 ffa500 ee82ee 9400d3 dda0dd 905040 556b2f
		801400 801414 804014 804080 8060c0 8060ff 808000 ff8040 ffa040 ffa060
		ffa070 ffc0c0 ffff80 ffffc0 cdb79e f0fff0 a0b6cd c1ffc1 cdc0b0 7cff40
		a0ff20 bebebe d3d3d3 a0a0a0 a0b6cd 000000 1a1a1a 333333 4d4d4d 666666
		7f7f7f 999999 b3b3b3 cccccc e5e5e5);
	@color_list = map { s/(..)(..)(..)/hex($1).",".hex($2).",".hex($3)/e; $_ } @color_list;

	foreach my $uid (@uid_list) {
		my @data;
		my %time_list;
		my @qid_list = question::clear_list($uid);
		foreach my $qid (@qid_list) {
			my $t = question::get_time($uid, $qid);
			$t = int(($t + 59) / 60) * 60;
			$time_list{$t} = 1;
		}
		$time_list{$_ - 60} = 1 for (sort keys %time_list)[0];
		foreach my $t (sort keys %time_list) {
			push @data, "					{ t: '" . localtime($t) . "', y: " . question::total_score($uid, $t) . " }" ;
		}
		my $color = shift @color_list;
		my $data = "				{ label: '" . user::get_name($uid) . "', lineTension: 0, borderColor: 'rgba($color,0.8)', backgroundColor: 'rgba($color,0.4)', fill: false, data: [\n";
		$data .= join(",\n", @data) . "\n";
		$data .= "				]}";
		push @userdata, $data;
	}
	print join(",\n", @userdata), "\n";
	print <<"EOD";
			]
		},
		options: {
			scales: {
				xAxes: [{
					type: 'time',
					scaleLabel: {
						labelString: 'Time',
						display: true
					},
					time: {
						unit: 'minute',
						displayFormats: { minute: 'M/D H:mm' },
						tooltipFormat: 'M/D H:mm'
					}
				}],
				yAxes: [{
					scaleLabel: {
						labelString: 'Score',
						display: true
					}
				}]
			}
		}
	});
</script>
EOD

cgilib::footer($cgi);
