#!/usr/bin/perl

use strict;
use CGI;
use CGI::Session;
use POSIX;

require 'config.pl';
require 'lib/cgilib.pl';
require 'lib/session.pl';
require 'lib/user.pl';
require 'lib/question.pl';

my $cgi = CGI->new;
my $session_id;
my $session;

($session_id, $session) = session::check($cgi);

my $uid = $session->param('uid');
if (!user::is_valid($uid)) {
	cgilib::header($cgi, "Error");
	print <<"EOD";
<h1>Invalid User!</h1>
<p><a href="login.cgi">Login</a></p>
EOD
	cgilib::footer($cgi);
	exit;
}

my $name = user::get_name($uid);

if ($config::end_time le strftime "%Y-%m-%d %H:%M", localtime) {
	cgilib::header($cgi, "Closed");
	cgilib::menu($session_id, $name);
	print "<p>The competition has ended at $config::end_time.</p>\n";
	cgilib::footer($cgi);
	exit;
}

my $qid = $cgi->param('qid');
if (!question::is_valid($qid)) {
	cgilib::header($cgi, "Error");
	cgilib::menu($session_id, $name);
	print "<h1>Invalid Question!</h1>\n";
	cgilib::footer($cgi);
	exit;
}

my $reply = $cgi->param('reply');

if (question::is_clear($uid, $qid)) {
	cgilib::header($cgi, "Error");
	cgilib::menu($session_id, $name);
	print "<h1>Already Clear!</h1>\n";
	cgilib::footer($cgi);
	exit;
}

my $genre;
my $score;
my $title;
my $question;
my @answers;
my $answer;
my $correct;

($genre, $score, $title, $question, @answers) = question::readfile($qid);

$correct = 0;
foreach $answer (@answers) {
	if ($answer eq $reply) {
		$correct = 1;
		last;
	}
}

if (!$correct) {
	question::reply($uid, $qid, $reply);

	cgilib::header($cgi, "Incorrect");
	cgilib::menu($session_id, $name);
	print <<"EOD";
<h1>Bad Answer!</h1>
<p><a href="question.cgi?qid=$qid">Retry</a></p>
EOD
	cgilib::footer($cgi);
	exit;
}

cgilib::header($cgi, "Correct");
cgilib::menu($session_id, $name);
print "<h1>Correct!</h1>\n";
cgilib::footer($cgi);

question::clear($uid, $qid);
